LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY TestOpl IS
END TestOpl;

ARCHITECTURE behavior OF TestOpl IS
  COMPONENT MasterOpl
    PORT (
      rst : IN STD_LOGIC;
      clk : IN STD_LOGIC;
      en : IN STD_LOGIC;
      v1 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      v2 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      miso : IN STD_LOGIC;
      ss : OUT STD_LOGIC;
      sclk : OUT STD_LOGIC;
      mosi : OUT STD_LOGIC;
      val_nand : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      val_nor : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      val_xor : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      busy : OUT STD_LOGIC;
      debug_state : OUT NATURAL;
      debug_er_state : OUT NATURAL;
      er_busy : OUT STD_LOGIC;
      debug_cpt_wait : OUT NATURAL;
      debug_cpt_octet : OUT NATURAL
    );
  END COMPONENT;

  COMPONENT SlaveOpl
    PORT (
      sclk : IN STD_LOGIC;
      mosi : IN STD_LOGIC;
      ss : IN STD_LOGIC;
      miso : OUT STD_LOGIC
    );
  END COMPONENT;
  --Inputs
  SIGNAL rst : STD_LOGIC := '0';
  SIGNAL clk : STD_LOGIC := '0';
  SIGNAL en : STD_LOGIC := '0';
  SIGNAL v1 : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');
  SIGNAL v2 : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');
  SIGNAL miso : STD_LOGIC := '0';

  --Outputs
  SIGNAL ss : STD_LOGIC;
  SIGNAL sclk : STD_LOGIC;
  SIGNAL mosi : STD_LOGIC;
  SIGNAL val_nand : STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL val_nor : STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL val_xor : STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL busy : STD_LOGIC;
  SIGNAL debug_state : NATURAL;
  SIGNAL debug_er_state : NATURAL;
  SIGNAL debug_cpt_octet : NATURAL;
  SIGNAL debug_cpt_wait : NATURAL;

  SIGNAL er_busy : STD_LOGIC;

  -- Clock period definitions
  CONSTANT clk_period : TIME := 10 ns;

BEGIN

  -- Instantiate the Unit Under Test (UUT)
  uut : MasterOpl PORT MAP(
    rst => rst,
    clk => clk,
    en => en,
    v1 => v1,
    v2 => v2,
    miso => miso,
    ss => ss,
    sclk => sclk,
    mosi => mosi,
    val_nand => val_nand,
    val_nor => val_nor,
    val_xor => val_xor,
    busy => busy,
    debug_state => debug_state,
    debug_er_state => debug_er_state,
    er_busy => er_busy,
    debug_cpt_octet => debug_cpt_octet,
    debug_cpt_wait => debug_cpt_wait
  );

  Inst_SlaveOpl : SlaveOpl PORT MAP(
    sclk => sclk,
    mosi => mosi,
    miso => miso,
    ss => ss
  );
  -- Clock process definitions
  clk_process : PROCESS
  BEGIN
    clk <= '0';
    WAIT FOR clk_period/2;
    clk <= '1';
    WAIT FOR clk_period/2;
  END PROCESS;

  -- Stimulus process
  stim_proc : PROCESS
  BEGIN
    -- hold reset state for 100 ns.
    WAIT FOR 100 ns;

    WAIT FOR clk_period * 10;

    -- insert stimulus here 
    rst <= '1';
    en <= '1';

    -- insert stimulus here 
    v1 <= "00010101";
    v2 <= "10000100";

    WAIT FOR 1050 ns;

    v1 <= "00000001";
    v2 <= "00000001";

    WAIT FOR 1050 ns;

    v1 <= "11111111";
    v2 <= "00000000";

    WAIT FOR 1050 ns;

    v1 <= "01010100";
    v2 <= "10010010";
    WAIT FOR 1050 ns;

    v1 <= "01010101";
    v2 <= "10101010";

    WAIT FOR 1050 ns;

    v1 <= "11101101";
    v2 <= "01010110";

    WAIT;
  END PROCESS;

END;
